# Copyright 2018 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS-IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Define the static list of upgrades for SC2."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import enum


# pylint: disable=invalid-name
class Upgrades(enum.IntEnum):
  """The list of upgrades, as generated by bin/gen_data.py."""
  AdaptiveTalons = 0
  AdrenalGlands = 1
  AdvancedBallistics = 2
  AnabolicSynthesis = 3
  AnionPulseCrystals = 4
  Blink = 5
  Burrow = 6
  CentrificalHooks = 7
  Charge = 8
  ChitinousPlating = 9
  CloakingField = 10
  CombatShield = 11
  ConcussiveShells = 12
  CorvidReactor = 13
  CycloneRapidFireLaunchers = 14
  DrillingClaws = 15
  EnhancedShockwaves = 16
  ExtendedThermalLance = 17
  GlialReconstitution = 18
  GraviticBooster = 19
  GraviticDrive = 20
  GravitonCatapult = 21
  GroovedSpines = 22
  HiSecAutoTracking = 23
  HighCapacityFuelTanks = 24
  HyperflightRotors = 25
  InfernalPreigniter = 26
  LockOn = 27
  MetabolicBoost = 28
  MuscularAugments = 29
  NeosteelFrame = 30
  NeuralParasite = 31
  PathogenGlands = 32
  PersonalCloaking = 33
  PneumatizedCarapace = 34
  ProtossAirArmorsLevel1 = 35
  ProtossAirArmorsLevel2 = 36
  ProtossAirArmorsLevel3 = 37
  ProtossAirWeaponsLevel1 = 38
  ProtossAirWeaponsLevel2 = 39
  ProtossAirWeaponsLevel3 = 40
  ProtossGroundArmorsLevel1 = 41
  ProtossGroundArmorsLevel2 = 42
  ProtossGroundArmorsLevel3 = 43
  ProtossGroundWeaponsLevel1 = 44
  ProtossGroundWeaponsLevel2 = 45
  ProtossGroundWeaponsLevel3 = 46
  ProtossShieldsLevel1 = 47
  ProtossShieldsLevel2 = 48
  ProtossShieldsLevel3 = 49
  PsiStorm = 50
  ResonatingGlaives = 51
  ShadowStrike = 52
  SmartServos = 53
  Stimpack = 54
  TerranInfantryArmorsLevel1 = 55
  TerranInfantryArmorsLevel2 = 56
  TerranInfantryArmorsLevel3 = 57
  TerranInfantryWeaponsLevel1 = 58
  TerranInfantryWeaponsLevel2 = 59
  TerranInfantryWeaponsLevel3 = 60
  TerranShipWeaponsLevel1 = 61
  TerranShipWeaponsLevel2 = 62
  TerranShipWeaponsLevel3 = 63
  TerranStructureArmor = 64
  TerranVehicleAndShipArmorsLevel1 = 65
  TerranVehicleAndShipArmorsLevel2 = 66
  TerranVehicleAndShipArmorsLevel3 = 67
  TerranVehicleWeaponsLevel1 = 68
  TerranVehicleWeaponsLevel2 = 69
  TerranVehicleWeaponsLevel3 = 70
  TunnelingClaws = 71
  WarpGateResearch = 72
  WeaponRefit = 73
  ZergFlyerArmorsLevel1 = 74
  ZergFlyerArmorsLevel2 = 75
  ZergFlyerArmorsLevel3 = 76
  ZergFlyerWeaponsLevel1 = 77
  ZergFlyerWeaponsLevel2 = 78
  ZergFlyerWeaponsLevel3 = 79
  ZergGroundArmorsLevel1 = 80
  ZergGroundArmorsLevel2 = 81
  ZergGroundArmorsLevel3 = 82
  ZergMeleeWeaponsLevel1 = 83
  ZergMeleeWeaponsLevel2 = 84
  ZergMeleeWeaponsLevel3 = 85
  ZergMissileWeaponsLevel1 = 86
  ZergMissileWeaponsLevel2 = 87
  ZergMissileWeaponsLevel3 = 88


def get_upgrade_type(name):
  print("name: " + str(name))

  try:
    return Upgrades[name]
  except ValueError:
    print("ValueError")
    pass  # Wrong race.